/* Copyright 2022, Michele Santullo
 * This file is part of duckticker.
 *
 * Wrenpp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Wrenpp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with duckticker.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "websocket_reader.hpp"
#include "bitcoinity_payload.hpp"
#include <stdexcept>
#include <vector>
#include <utility>

namespace duck {
class BitcoinityError : public std::runtime_error {
public:
	BitcoinityError(const std::string& message) : std::runtime_error(message) {}
private:
};

class BitcoinityMissingKeyError : public BitcoinityError {
public:
	BitcoinityMissingKeyError (const std::vector<std::string>& keys, const std::string& json);

	const std::vector<std::string>& keys() const noexcept;
	const std::string& json() const noexcept;

private:
	std::vector<std::string> m_keys;
	std::string m_json;
};

struct BitcoinityMessage {
	BitcoinityMessage (
		BitcoinityPayload&& payload_p,
		std::optional<std::string_view> ref_p,
		std::string_view event_p,
		std::string_view topic_p
	);

	BitcoinityPayload payload;
	std::optional<std::string_view> ref;
	std::string_view event;
	std::string_view topic;
};

class BitcoinityReader : private WebsocketReader {
public:
	BitcoinityReader(const std::string& user_agent, std::string_view exchange, std::string_view currency);
	~BitcoinityReader() = default;

	BitcoinityMessage read();
};

} //namespace duck
